@@ -4,11 +4,16 @@ See the file COPYING for license details.
 */
 
 
-#include "nids.h"
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <netinet/in_systm.h>
+#include <arpa/inet.h>
 #include <string.h>
 #include <stdio.h>
+#include "nids.h"
 
-extern char * inet_ntoa(unsigned long);
+#define int_ntoa(x)	inet_ntoa(*((struct in_addr *)&x))
 
 // struct tuple4 contains addresses and port numbers of the TCP connections
 // the following auxiliary function produces a string looking like
@@ -17,9 +22,9 @@ char *
 adres (struct tuple4 addr)
 {
   static char buf[256];
-  strcpy (buf, inet_ntoa (addr.saddr));
+  strcpy (buf, int_ntoa (addr.saddr));
   sprintf (buf + strlen (buf), ",%i,", addr.source);
-  strcat (buf, inet_ntoa (addr.daddr));
+  strcat (buf, int_ntoa (addr.daddr));
   sprintf (buf + strlen (buf), ",%i", addr.dest);
   return buf;
 }
