@@ -50,7 +50,7 @@ struct timer_list {
 struct hostfrags {
   struct ipq *ipqueue;
   int ip_frag_mem;
-  unsigned long ip;
+  u_int ip;
   int hash_index;
   struct hostfrags *prev;
   struct hostfrags *next;
