@@ -16,10 +16,6 @@ nids_killtcp(struct tcp_stream * a_tcp)
 {
   char buf[IP_H + TCP_H];
   
-  if (libnetsock <= 0)
-    libnetsock = libnet_open_raw_sock(IPPROTO_RAW);
-  if (libnetsock <= 0)
-    return;
   libnet_build_ip(TCP_H, 0, 12345, 0, 64, IPPROTO_TCP, a_tcp->addr.saddr,
 		  a_tcp->addr.daddr, 0, 0, buf);
   libnet_build_tcp(a_tcp->addr.source, a_tcp->addr.dest,
@@ -36,3 +32,10 @@ nids_killtcp(struct tcp_stream * a_tcp)
   libnet_do_checksum(buf, IPPROTO_TCP, TCP_H);
   libnet_write_ip(libnetsock, buf, TCP_H + IP_H);
 }
+int raw_init()
+{
+	libnetsock = libnet_open_raw_sock(IPPROTO_RAW);
+	if (libnetsock <= 0)
+	    return 0;
+	else return 1;
+}
