@@ -73,7 +73,8 @@ struct nids_prm nids_params = {
   nids_no_mem,		/* no_mem() */
   nids_ip_filter,	/* ip_filter() */
   NULL,			/* pcap_filter */
-  1                     /* promisc */
+  1,                    /* promisc */
+  0			/* one_loop_less */
 };
 
 static int
@@ -191,7 +192,7 @@ gen_ip_frag_proc(u_char *data, int len)
     return;
   }
   if (iph->ip_hl > 5 && ip_options_compile(data)) {
-    nids_params.syslog(NIDS_WARN_IP, NIDS_WARN_IP_SRR, iph->ip_hl, 0);
+    nids_params.syslog(NIDS_WARN_IP, NIDS_WARN_IP_SRR, iph, 0);
     return;
   }
   switch (ip_defrag_stub((struct ip *)data, &iph)) {
@@ -340,8 +341,8 @@ nids_init()
 	return 0;
       if (nids_params.promisc && !set_all_promisc())
       {
-      	strncpy(nids_errbuf, strerror(errno), sizeof(nids_errbuf));
-      	nids_errbuf[sizeof(nids_errbuf)-1]=0;
+        nids_errbuf[0]=0;
+        strncat(nids_errbuf, strerror(errno), sizeof(nids_errbuf) - 1);
       	return 0;
       }	
   #else
@@ -374,6 +375,11 @@ nids_init()
   case DLT_PPP:
     linkoffset = 4;
     break;
+  /* Token Ring Support by vacuum@technotronic.com, thanks dugsong! */
+   case DLT_IEEE802:
+     linkoffset = 22;
+     break;
+    
   case DLT_RAW:
   case DLT_SLIP:
     linkoffset = 0;
@@ -399,8 +405,8 @@ nids_init()
   scan_init();
   if (!raw_init())
   {
-        strncpy(nids_errbuf, strerror(errno), sizeof(nids_errbuf));
-        nids_errbuf[sizeof(nids_errbuf)-1]=0;
+  	nids_errbuf[0]=0;
+        strncat(nids_errbuf, strerror(errno), sizeof(nids_errbuf) - 1);
         return 0;
   }  	                           
   return 1;
@@ -414,9 +420,8 @@ nids_run()
     return;
   }
   pcap_loop(desc, -1, (pcap_handler) pcap_hand, 0);
-  strcpy(nids_errbuf, "loop:");
-  strncpy(nids_errbuf + 5, pcap_geterr(desc), sizeof(nids_errbuf - 7));
-  nids_errbuf[sizeof(nids_errbuf) - 1] = 0;
+  strcpy(nids_errbuf, "loop: ");
+  strncat(nids_errbuf, pcap_geterr(desc), sizeof(nids_errbuf) - 7);
   pcap_close(desc);
 }
 
@@ -441,9 +446,8 @@ nids_next()
     return 0;
   }
   if (!(data = (char *) pcap_next(desc, &h))) {
-    strcpy(nids_errbuf, "next:");
-    strncpy(nids_errbuf + 5, pcap_geterr(desc), sizeof(nids_errbuf - 7));
-    nids_errbuf[sizeof(nids_errbuf) - 1] = 0;
+    strcpy(nids_errbuf, "next: ");
+    strncat(nids_errbuf, pcap_geterr(desc), sizeof(nids_errbuf) - 7);
     return 0;
   }
   pcap_hand(0, &h, data);
