@@ -233,8 +233,7 @@ ride_lurkers(struct tcp_stream * a_tcp, char mask)
 }
 
 static void
-notify(struct tcp_stream * a_tcp, struct half_stream * rcv,
-       struct half_stream * snd)
+notify(struct tcp_stream * a_tcp, struct half_stream * rcv)
 {
   struct lurker_node *i, **prev_addr;
   char mask;
@@ -249,20 +248,27 @@ notify(struct tcp_stream * a_tcp, struct half_stream * rcv,
     ride_lurkers(a_tcp, mask);
     goto prune_listeners;
   }
-  a_tcp->read = rcv->count - rcv->offset;
   if (rcv->collect) {
     if (rcv == &a_tcp->client)
       mask = COLLECT_cc;
     else
       mask = COLLECT_sc;
-
-    ride_lurkers(a_tcp, mask);
-
-    rcv->count_new = 0;
-    if (a_tcp->read > 0) {
-      memmove(rcv->data, rcv->data + a_tcp->read, rcv->count - rcv->offset - a_tcp->read);
-      rcv->offset += a_tcp->read;
-    }
+   do {
+	int total;
+		a_tcp->read = rcv->count - rcv->offset;
+		  total=a_tcp->read;
+  
+	    ride_lurkers(a_tcp, mask);
+	    if (a_tcp->read>total-rcv->count_new)
+	    	rcv->count_new=total-a_tcp->read;
+	    
+	    if (a_tcp->read > 0) {
+	      memmove(rcv->data, rcv->data + a_tcp->read, rcv->count - rcv->offset - a_tcp->read);
+	      rcv->offset += a_tcp->read;
+	    }
+	}while (nids_params.one_loop_less && a_tcp->read>0 && rcv->count_new); 
+// we know that if one_loop_less!=0, we have only one callback to notify
+   rcv->count_new=0;	    
   }
  prune_listeners:
   prev_addr = &a_tcp->listeners;
@@ -299,7 +305,7 @@ add_from_skb(struct tcp_stream * a_tcp, struct half_stream * rcv,
     if (to_copy > 0) {
       if (rcv->collect) {
 	add2buf(rcv, data + lost, to_copy);
-	notify(a_tcp, rcv, snd);
+	notify(a_tcp, rcv);
       }
       else {
 	rcv->count += to_copy;
@@ -307,14 +313,14 @@ add_from_skb(struct tcp_stream * a_tcp, struct half_stream * rcv,
     }
     rcv->urgdata = data[rcv->urg_ptr - this_seq];
     rcv->count_new_urg = 1;
-    notify(a_tcp, rcv, snd);
+    notify(a_tcp, rcv);
     rcv->count_new_urg = 0;
     rcv->urg_count++;
     to_copy2 = this_seq + datalen - rcv->urg_ptr - 1;
     if (to_copy2 > 0) {
       if (rcv->collect) {
 	add2buf(rcv, data + lost + to_copy + 1, to_copy2);
-	notify(a_tcp, rcv, snd);
+	notify(a_tcp, rcv);
       }
       else {
 	rcv->count += to_copy2;
@@ -325,7 +331,7 @@ add_from_skb(struct tcp_stream * a_tcp, struct half_stream * rcv,
     if (datalen - lost > 0) {
       if (rcv->collect) {
 	add2buf(rcv, data + lost, datalen - lost);
-	notify(a_tcp, rcv, snd);
+	notify(a_tcp, rcv);
       }
       else {
 	rcv->count += datalen - lost;
@@ -617,6 +623,16 @@ process_tcp(u_char * data, int skblen)
 	      whatto |= COLLECT_sc;
 	    if (scu < a_tcp->server.collect_urg)
 	      whatto |= COLLECT_scu;
+	    if (nids_params.one_loop_less) {
+	    		if (a_tcp->client.collect >=2) {
+	    			a_tcp->client.collect=cc;
+	    			whatto&=~COLLECT_cc;
+	    		}
+	    		if (a_tcp->server.collect >=2 ) {
+	    			a_tcp->server.collect=sc;
+	    			whatto&=~COLLECT_sc;
+	    		}
+	    }  
 	    if (whatto) {
 	      j = mknew(struct lurker_node);
 	      j->item = i->item;
